@@ -27,7 +27,6 @@ __license__   = "LGPL"
 
 import pyatspi
 import urllib.parse, urllib.request, urllib.error, urllib.parse
-from gi.repository import Atspi, Atk
 
 from . import chnames
 from . import debug
@@ -210,6 +209,9 @@ class SpeechGenerator(generator.Generator):
         if _settingsManager.getSetting('onlySpeakDisplayedText'):
             return []
 
+        if not _settingsManager.getSetting('speakDescription'):
+            return []
+
         priorObj = args.get('priorObj')
         if priorObj and priorObj.getRole() == pyatspi.ROLE_TOOL_TIP:
             return []
@@ -220,6 +222,22 @@ class SpeechGenerator(generator.Generator):
             result.extend(acss)
         return result
 
+    def _generateImageDescription(self, obj, **args ):
+        """Returns an array of strings for use by speech and braille that
+        represent the description of the image on the object."""
+
+        if _settingsManager.getSetting('onlySpeakDisplayedText'):
+            return []
+
+        if not _settingsManager.getSetting('speakDescription'):
+            return []
+
+        acss = self.voice(SYSTEM)
+        result = generator.Generator._generateImageDescription(self, obj, **args)
+        if result:
+            result.extend(acss)
+        return result
+
     def _generateReadOnly(self, obj, **args):
         """Returns an array of strings for use by speech and braille that
         represent the read only state of this object, but only if it
@@ -263,6 +281,16 @@ class SpeechGenerator(generator.Generator):
             result.extend(acss)
         return result
 
+    def _generateInvalid(self, obj, **args):
+        if _settingsManager.getSetting('onlySpeakDisplayedText'):
+            return []
+
+        acss = self.voice(SYSTEM)
+        result = generator.Generator._generateInvalid(self, obj, **args)
+        if result:
+            result.extend(acss)
+        return result
+
     def _generateRequired(self, obj, **args):
         if _settingsManager.getSetting('onlySpeakDisplayedText'):
             return []
@@ -277,7 +305,15 @@ class SpeechGenerator(generator.Generator):
         if _settingsManager.getSetting('onlySpeakDisplayedText'):
             return []
 
-        if _settingsManager.getSetting('speechVerbosityLevel') == \
+        if args.get("leaving"):
+            return[]
+
+        if self._script.utilities.isTextDocumentTable(obj):
+            role = args.get('role', obj.getRole())
+            enabled, disabled = self._getEnabledAndDisabledContextRoles()
+            if role in disabled:
+                return []
+        elif _settingsManager.getSetting('speechVerbosityLevel') == \
            settings.VERBOSITY_LEVEL_BRIEF:
             return []
 
@@ -349,12 +385,12 @@ class SpeechGenerator(generator.Generator):
             level = self._script.utilities.headingLevel(obj)
             if level:
                 result.append(object_properties.ROLE_HEADING_LEVEL_SPEECH % {
-                    'role': self.getLocalizedRoleName(obj, role),
+                    'role': self.getLocalizedRoleName(obj, **args),
                     'level': level})
                 result.extend(acss)
 
         if role not in doNotPresent and not result:
-            result.append(self.getLocalizedRoleName(obj, role))
+            result.append(self.getLocalizedRoleName(obj, **args))
             result.extend(acss)
         return result
 
@@ -366,33 +402,37 @@ class SpeechGenerator(generator.Generator):
         accessible role of the obj.  This is provided mostly as a
         method for scripts to call.
         """
-        return self._generateRoleName(obj, **args)
+        generated = self._generateRoleName(obj, **args)
+        if generated:
+            return generated[0]
 
-    def getLocalizedRoleName(self, obj, role=None):
+        return ""
+
+    def getName(self, obj, **args):
+        generated = self._generateName(obj, **args)
+        if generated:
+            return generated[0]
+
+        return ""
+
+    def getLocalizedRoleName(self, obj, **args):
         """Returns the localized name of the given Accessible object; the name
         is suitable to be spoken.
 
         Arguments:
         - obj: an Accessible object
-        - role: an optional pyatspi role to use instead
         """
 
-        if not isinstance(role, pyatspi.Role):
-            try:
-                return obj.getLocalizedRoleName()
-            except:
-                return ''
+        if self._script.utilities.isEditableComboBox(obj) \
+           or self._script.utilities.isEditableDescendantOfComboBox(obj):
+            return object_properties.ROLE_EDITABLE_COMBO_BOX
 
-        if not role:
-            return ''
-
-        if role == pyatspi.ROLE_LINK and obj.getState().contains(pyatspi.STATE_VISITED):
+        role = args.get('role', obj.getRole())
+        state = obj.getState()
+        if role == pyatspi.ROLE_LINK and state.contains(pyatspi.STATE_VISITED):
             return object_properties.ROLE_VISITED_LINK
 
-        nonlocalized = Atspi.role_get_name(role)
-        atkRole = Atk.role_for_name(nonlocalized)
-
-        return Atk.role_get_localized_name(atkRole)
+        return super().getLocalizedRoleName(obj, **args)
 
     def _generateUnrelatedLabels(self, obj, **args):
         """Returns, as an array of strings (and possibly voice
@@ -492,6 +532,17 @@ class SpeechGenerator(generator.Generator):
             result.extend(acss)
         return result
 
+    def _generateSwitchState(self, obj, **args):
+        """Returns an array of strings indicating the on/off state of obj."""
+        if _settingsManager.getSetting('onlySpeakDisplayedText'):
+            return []
+
+        acss = self.voice(STATE)
+        result = generator.Generator._generateSwitchState(self, obj, **args)
+        if result:
+            result.extend(acss)
+        return result
+
     def _generateToggleState(self, obj, **args):
         """Returns an array of strings for use by speech and braille that
         represent the checked state of the object.  This is typically
@@ -683,44 +734,11 @@ class SpeechGenerator(generator.Generator):
         dictionary.  The 'priorObj' is typically set by Orca to be the
         previous object with focus.
         """
-        result = []
-        acss = self.voice(DEFAULT)
-        if obj:
-            priorObj = args.get('priorObj', None)
-            try:
-                priorParent = priorObj.parent
-            except:
-                priorParent = None
 
-            if (obj.getRole() == pyatspi.ROLE_TABLE_CELL) \
-                or (obj.parent and obj.parent.getRole() == pyatspi.ROLE_TABLE):
-                try:
-                    table = priorParent.queryTable()
-                except:
-                    table = None
-                if table \
-                   and ((priorObj.getRole() == pyatspi.ROLE_TABLE_CELL) \
-                         or (priorObj.getRole() == pyatspi.ROLE_TABLE)):
-                    index = self._script.utilities.cellIndex(priorObj)
-                    oldRow = table.getRowAtIndex(index)
-                else:
-                    oldRow = -1
+        if not self._script.utilities.cellRowChanged(obj):
+            return []
 
-                try:
-                    table = obj.parent.queryTable()
-                except:
-                    pass
-                else:
-                    index = self._script.utilities.cellIndex(obj)
-                    newRow = table.getRowAtIndex(index)
-                    if (newRow >= 0) \
-                       and (index != newRow) \
-                       and ((newRow != oldRow) \
-                            or (obj.parent != priorParent)):
-                        result = self._generateRowHeader(obj, **args)
-        if result:
-            result.extend(acss)
-        return result
+        return self._generateRowHeader(obj, **args)
 
     def _generateNewColumnHeader(self, obj, **args):
         """Returns an array of strings (and possibly voice and audio
@@ -732,44 +750,14 @@ class SpeechGenerator(generator.Generator):
         dictionary.  The 'priorObj' is typically set by Orca to be the
         previous object with focus.
         """
-        result = []
-        acss = self.voice(DEFAULT)
-        if obj and not args.get('readingRow', False):
-            priorObj = args.get('priorObj', None)
-            try:
-                priorParent = priorObj.parent
-            except:
-                priorParent = None
 
-            if (obj.getRole() == pyatspi.ROLE_TABLE_CELL) \
-                or (obj.parent and obj.parent.getRole() == pyatspi.ROLE_TABLE):
-                try:
-                    table = priorParent.queryTable()
-                except:
-                    table = None
-                if table \
-                   and ((priorObj.getRole() == pyatspi.ROLE_TABLE_CELL) \
-                         or (priorObj.getRole() == pyatspi.ROLE_TABLE)):
-                    index = self._script.utilities.cellIndex(priorObj)
-                    oldCol = table.getColumnAtIndex(index)
-                else:
-                    oldCol = -1
+        if not self._script.utilities.cellColumnChanged(obj):
+            return []
 
-                try:
-                    table = obj.parent.queryTable()
-                except:
-                    pass
-                else:
-                    index = self._script.utilities.cellIndex(obj)
-                    newCol = table.getColumnAtIndex(index)
-                    if (newCol >= 0) \
-                       and (index != newCol) \
-                       and ((newCol != oldCol) \
-                            or (obj.parent != priorParent)):
-                        result = self._generateColumnHeader(obj, **args)
-        if result:
-            result.extend(acss)
-        return result
+        if args.get('readingRow'):
+            return []
+
+        return self._generateColumnHeader(obj, **args)
 
     def _generateRealTableCell(self, obj, **args):
         """Orca has a feature to automatically read an entire row of a table
@@ -955,7 +943,8 @@ class SpeechGenerator(generator.Generator):
         acss = self.voice(DEFAULT)
         result = generator.Generator._generateCurrentLineText(self, obj, **args)
         if result:
-            if result == ["\n"]:
+            if result == ['\n'] and _settingsManager.getSetting('speakBlankLines') \
+               and not self._script.inSayAll() and args.get('total', 1) == 1:
                 result = [messages.BLANK]
             result.extend(acss)
         return result
@@ -1068,9 +1057,8 @@ class SpeechGenerator(generator.Generator):
             else:
                 char = textObj.getTextAtOffset(caretOffset,
                     pyatspi.TEXT_BOUNDARY_CHAR)
-                if char[0] == "\n" and startOffset == caretOffset \
-                       and _settingsManager.getSetting('speakBlankLines'):
-                    textContents = (messages.BLANK)
+                if char[0] == "\n" and startOffset == caretOffset:
+                    textContents = char[0]
 
         self._script.generatorCache['textInformation'] = \
             [textContents, startOffset, endOffset, selected]
@@ -1211,6 +1199,10 @@ class SpeechGenerator(generator.Generator):
             result[0] = result[0].strip()
 
         result.extend(self._getACSS(obj, result[0]))
+        if result[0] in ['\n', ''] and _settingsManager.getSetting('speakBlankLines') \
+           and not self._script.inSayAll() and args.get('total', 1) == 1:
+            result[0] = messages.BLANK
+
         return result
 
     def _generateTextIndentation(self, obj, **args):
@@ -1277,7 +1269,7 @@ class SpeechGenerator(generator.Generator):
 
         percentValue = self._script.utilities.getValueAsPercent(obj)
         if percentValue is not None:
-            result = [percentValue]
+            result = [messages.percentage(percentValue)]
             result.extend(self.voice(SYSTEM))
             return result
 
@@ -1402,8 +1394,7 @@ class SpeechGenerator(generator.Generator):
         if not (items and items[0]):
             return result
 
-        items = list(map(self._generateName, items))
-        for item in items:
+        for item in map(self._generateName, items):
             result.extend(item)
 
         return result
@@ -1482,6 +1473,97 @@ class SpeechGenerator(generator.Generator):
             result.extend(acss)
         return result
 
+    def _getEnabledAndDisabledContextRoles(self):
+        allRoles = [pyatspi.ROLE_BLOCK_QUOTE,
+                    pyatspi.ROLE_FORM,
+                    pyatspi.ROLE_LANDMARK,
+                    pyatspi.ROLE_LIST,
+                    pyatspi.ROLE_PANEL,
+                    pyatspi.ROLE_TABLE]
+
+        enabled, disabled = [], []
+        if self._script.inSayAll():
+            if _settingsManager.getSetting('sayAllContextBlockquote'):
+                enabled.append(pyatspi.ROLE_BLOCK_QUOTE)
+            if _settingsManager.getSetting('sayAllContextLandmark'):
+                enabled.append(pyatspi.ROLE_LANDMARK)
+            if _settingsManager.getSetting('sayAllContextList'):
+                enabled.append(pyatspi.ROLE_LIST)
+            if _settingsManager.getSetting('sayAllContextPanel'):
+                enabled.append(pyatspi.ROLE_PANEL)
+            if _settingsManager.getSetting('sayAllContextNonLandmarkForm'):
+                enabled.append(pyatspi.ROLE_FORM)
+            if _settingsManager.getSetting('sayAllContextTable'):
+                enabled.append(pyatspi.ROLE_TABLE)
+        else:
+            if _settingsManager.getSetting('speakContextBlockquote'):
+                enabled.append(pyatspi.ROLE_BLOCK_QUOTE)
+            if _settingsManager.getSetting('speakContextLandmark'):
+                enabled.append(pyatspi.ROLE_LANDMARK)
+            if _settingsManager.getSetting('speakContextList'):
+                enabled.append(pyatspi.ROLE_LIST)
+            if _settingsManager.getSetting('speakContextPanel'):
+                enabled.append(pyatspi.ROLE_PANEL)
+            if _settingsManager.getSetting('speakContextNonLandmarkForm'):
+                enabled.append(pyatspi.ROLE_FORM)
+            if _settingsManager.getSetting('speakContextTable'):
+                enabled.append(pyatspi.ROLE_TABLE)
+
+        disabled = list(set(allRoles).symmetric_difference(enabled))
+        return enabled, disabled
+
+    def _generateLeaving(self, obj, **args):
+        if not args.get('leaving'):
+            return []
+
+        role = args.get('role', obj.getRole())
+        enabled, disabled = self._getEnabledAndDisabledContextRoles()
+        if not role in enabled:
+            return []
+
+        count = args.get('count', 1)
+
+        result = []
+        if role == pyatspi.ROLE_BLOCK_QUOTE:
+            if count > 1:
+                result.append(messages.leavingNBlockquotes(count))
+            else:
+                result.append(messages.LEAVING_BLOCKQUOTE)
+        elif role == pyatspi.ROLE_LIST and self._script.utilities.isDocumentList(obj):
+            if count > 1:
+                result.append(messages.leavingNLists(count))
+            else:
+                result.append(messages.LEAVING_LIST)
+        elif role == pyatspi.ROLE_PANEL and self._script.utilities.isDocumentPanel(obj):
+            result.append(messages.LEAVING_PANEL)
+        elif role == pyatspi.ROLE_TABLE and self._script.utilities.isTextDocumentTable(obj):
+            result.append(messages.LEAVING_TABLE)
+        elif self._script.utilities.isLandmark(obj):
+            if self._script.utilities.isLandmarkBanner(obj):
+                result.append(messages.LEAVING_LANDMARK_BANNER)
+            if self._script.utilities.isLandmarkComplementary(obj):
+                result.append(messages.LEAVING_LANDMARK_COMPLEMENTARY)
+            if self._script.utilities.isLandmarkContentInfo(obj):
+                result.append(messages.LEAVING_LANDMARK_CONTENTINFO)
+            if self._script.utilities.isLandmarkMain(obj):
+                result.append(messages.LEAVING_LANDMARK_MAIN)
+            if self._script.utilities.isLandmarkNavigation(obj):
+                result.append(messages.LEAVING_LANDMARK_NAVIGATION)
+            if self._script.utilities.isLandmarkRegion(obj):
+                result.append(messages.LEAVING_LANDMARK_REGION)
+            if self._script.utilities.isLandmarkSearch(obj):
+                result.append(messages.LEAVING_LANDMARK_SEARCH)
+            if self._script.utilities.isLandmarkForm(obj):
+                result.append(messages.LEAVING_FORM)
+        elif role == pyatspi.ROLE_FORM:
+            result.append(messages.LEAVING_FORM)
+        else:
+            result = ['']
+        if result:
+            result.extend(self.voice(SYSTEM))
+
+        return result
+
     def _generateAncestors(self, obj, **args):
         """Returns an array of strings (and possibly voice and audio
         specifications) that represent the text of the ancestors for
@@ -1494,44 +1576,85 @@ class SpeechGenerator(generator.Generator):
         previous object with focus.
         """
         result = []
-        priorObj = args.get('priorObj', None)
+
+        leaving = args.get('leaving')
+        if leaving and args.get('priorObj'):
+              priorObj = obj
+              obj = args.get('priorObj')
+        else:
+              priorObj = args.get('priorObj')
+
         if priorObj and priorObj.getRole() == pyatspi.ROLE_TOOL_TIP:
             return []
 
-        commonAncestor = self._script.utilities.commonAncestor(priorObj, obj)
-        try:
-            role = commonAncestor.getRole()
-            name = commonAncestor.name
-        except:
-            role = None
-            name = None
-        else:
-            if role == pyatspi.ROLE_COMBO_BOX:
-                return []
+        if priorObj and priorObj.parent == obj.parent:
+            return []
 
-        def _isCommonAncestor(x):
-            if not (name and role):
-                return False
-            return x and x.getRole() == role and x.name == name
+        if self._script.utilities.isTypeahead(priorObj):
+            return []
+
+        commonAncestor = self._script.utilities.commonAncestor(priorObj, obj)
+        if obj == commonAncestor:
+            return []
+
+        includeOnly = args.get('includeOnly', [])
 
         skipRoles = args.get('skipRoles', [])
         skipRoles.append(pyatspi.ROLE_TREE_ITEM)
-        stopAtRoles = args.get('stopAtRoles', [])
-        stopAtRoles.append(pyatspi.ROLE_APPLICATION)
-        if obj != commonAncestor:
-            parent = obj.parent
-            while parent and not parent in [commonAncestor, parent.parent]:
-                if _isCommonAncestor(parent):
-                    break
+        enabled, disabled = self._getEnabledAndDisabledContextRoles()
+        skipRoles.extend(disabled)
 
-                parentRole = parent.getRole()
-                if parentRole in stopAtRoles:
-                    break
-                if parentRole not in skipRoles \
-                   and not self._script.utilities.isLayoutOnly(parent):
-                    result.append(self.generate(parent, formatType='focused'))
-                parent = parent.parent
-        result.reverse()
+        stopAtRoles = args.get('stopAtRoles', [])
+        stopAtRoles.extend([pyatspi.ROLE_APPLICATION, pyatspi.ROLE_MENU_BAR])
+
+        presentOnce = [pyatspi.ROLE_BLOCK_QUOTE, pyatspi.ROLE_LIST]
+
+        presentCommonAncestor = False
+        if commonAncestor and not leaving:
+            commonRole = self._getAlternativeRole(commonAncestor)
+            if commonRole in presentOnce:
+                pred = lambda x: x and self._getAlternativeRole(x) == commonRole
+                objAncestor = pyatspi.findAncestor(obj, pred)
+                priorAncestor = pyatspi.findAncestor(priorObj, pred)
+                objLevel = self._script.utilities.nestingLevel(objAncestor)
+                priorLevel = self._script.utilities.nestingLevel(priorAncestor)
+                presentCommonAncestor = objLevel != priorLevel
+
+        ancestors, ancestorRoles = [], []
+        parent = obj.parent
+        while parent and parent != parent.parent:
+            parentRole = self._getAlternativeRole(parent)
+            if parentRole in stopAtRoles:
+                break
+            if parentRole in skipRoles:
+                pass
+            elif includeOnly and parentRole not in includeOnly:
+                pass
+            elif self._script.utilities.isLayoutOnly(parent):
+                pass
+            elif parent != commonAncestor or presentCommonAncestor:
+                ancestors.append(parent)
+                ancestorRoles.append(parentRole)
+
+            if parent == commonAncestor:
+                break
+
+            parent = parent.parent
+
+        presentedRoles = []
+        for i, x in enumerate(ancestors):
+            altRole = ancestorRoles[i]
+            if altRole in presentOnce and altRole in presentedRoles:
+                continue
+
+            presentedRoles.append(altRole)
+            count = ancestorRoles.count(altRole)
+            self._overrideRole(altRole, args)
+            result.append(self.generate(x, formatType='focused', role=altRole, leaving=leaving, count=count))
+            self._restoreRole(altRole, args)
+
+        if not leaving:
+            result.reverse()
         return result
 
     def _generateOldAncestors(self, obj, **args):
@@ -1539,7 +1662,41 @@ class SpeechGenerator(generator.Generator):
         specifications) that represent the text of the ancestors for
         the object being left."""
 
-        return []
+        if _settingsManager.getSetting('onlySpeakDisplayedText'):
+            return []
+
+        if self._script.utilities.inFindToolbar():
+            return []
+
+        priorObj = args.get('priorObj')
+        if not priorObj or obj == priorObj:
+            return []
+
+        if obj.getApplication() != priorObj.getApplication() \
+           or pyatspi.findAncestor(obj, lambda x: x == priorObj):
+            return []
+
+        args['leaving'] = True
+        args['includeOnly'] = [pyatspi.ROLE_BLOCK_QUOTE,
+                               pyatspi.ROLE_FORM,
+                               pyatspi.ROLE_LANDMARK,
+                               pyatspi.ROLE_LIST,
+                               pyatspi.ROLE_PANEL,
+                               pyatspi.ROLE_TABLE]
+
+        result = []
+        if self._script.utilities.isBlockquote(priorObj):
+            oldRole = self._getAlternativeRole(priorObj)
+            self._overrideRole(oldRole, args)
+            result.extend(self.generate(
+                priorObj, role=oldRole, formatType='focused', leaving=True))
+            self._restoreRole(oldRole, args)
+
+        result.extend(self._generateAncestors(obj, **args))
+        args.pop('leaving')
+        args.pop('includeOnly')
+
+        return result
 
     def _generateNewAncestors(self, obj, **args):
         """Returns an array of strings (and possibly voice and audio
@@ -1554,15 +1711,17 @@ class SpeechGenerator(generator.Generator):
         with focus.
         """
 
-        # TODO - JD: This is not the right way to do this, but we can fix
-        # that as part of the removal of formatting strings.
-        start = args.get('startOffset')
-        end = args.get('endOffset')
-        if start is not None or end is not None:
+        if _settingsManager.getSetting('onlySpeakDisplayedText'):
+            return []
+
+        if self._script.utilities.inFindToolbar():
+            return []
+
+        priorObj = args.get('priorObj')
+        if priorObj == obj:
             return []
 
         result = []
-        priorObj = args.get('priorObj')
         if obj.getRole() == pyatspi.ROLE_MENU_ITEM \
            and (not priorObj or priorObj.getRole() == pyatspi.ROLE_WINDOW):
             return result
@@ -1573,6 +1732,14 @@ class SpeechGenerator(generator.Generator):
             result = self._generateAncestors(obj, **args)
         return result
 
+    def generateContext(self, obj, **args):
+        if args.get('priorObj') == obj:
+            return []
+
+        result = self._generateOldAncestors(obj, **args)
+        result.append(self._generateNewAncestors(obj, **args))
+        return result
+
     def _generateParentRoleName(self, obj, **args):
         """Returns an array of strings (and possibly voice and audio
         specifications) containing the role name of the parent of obj.
@@ -1652,6 +1819,9 @@ class SpeechGenerator(generator.Generator):
            and obj.parent == self._script.utilities.topLevelObject(obj):
             return []
 
+        if self._script.utilities.isEditableComboBox(obj):
+            return []
+
         result = []
         acss = self.voice(SYSTEM)
         position, total = self._script.utilities.getPositionAndSetSize(obj)
@@ -1669,13 +1839,13 @@ class SpeechGenerator(generator.Generator):
 
     def _generateProgressBarIndex(self, obj, **args):
         if not args.get('isProgressBarUpdate') \
-           or not _settingsManager.getSetting('speakProgressBarUpdates'):
+           or not self._shouldPresentProgressBarUpdate(obj, **args):
             return []
 
         result = []
-        acc, updateTime, updateValue = self._script.utilities.getMostRecentProgressBarUpdate()
+        acc, updateTime, updateValue = self._getMostRecentProgressBarUpdate()
         if acc != obj:
-            number, count = self._script.utilities.getProgressBarNumberAndCount(obj)
+            number, count = self.getProgressBarNumberAndCount(obj)
             result = [messages.PROGRESS_BAR_NUMBER % (number)]
             result.extend(self.voice(SYSTEM))
 
@@ -1683,7 +1853,7 @@ class SpeechGenerator(generator.Generator):
 
     def _generateProgressBarValue(self, obj, **args):
         if args.get('isProgressBarUpdate') \
-           and not _settingsManager.getSetting('speakProgressBarUpdates'):
+           and not self._shouldPresentProgressBarUpdate(obj, **args):
             return []
 
         result = []
@@ -1694,6 +1864,19 @@ class SpeechGenerator(generator.Generator):
 
         return result
 
+    def _getProgressBarUpdateInterval(self):
+        interval = _settingsManager.getSetting('progressBarSpeechInterval')
+        if interval is None:
+            interval = super()._getProgressBarUpdateInterval()
+
+        return int(interval)
+
+    def _shouldPresentProgressBarUpdate(self, obj, **args):
+        if not _settingsManager.getSetting('speakProgressBarUpdates'):
+            return False
+
+        return super()._shouldPresentProgressBarUpdate(obj, **args)
+
     def _generateDefaultButton(self, obj, **args):
         """Returns an array of strings (and possibly voice and audio
         specifications) that represent the default button in a dialog.
@@ -1705,6 +1888,8 @@ class SpeechGenerator(generator.Generator):
             name = self._generateName(button)
             if name:
                 result.append(messages.DEFAULT_BUTTON_IS % name[0])
+                result.extend(self.voice(SYSTEM))
+
         return result
 
     def generateDefaultButton(self, obj, **args):
@@ -2233,14 +2418,15 @@ class SpeechGenerator(generator.Generator):
 
         voicename = voiceType.get(key) or voiceType.get(DEFAULT)
         voices = _settingsManager.getSetting('voices')
-        rv = voices.get(voicename)
-        if not rv or rv.get('established') == False:
-            rv = voices.get(voiceType.get(DEFAULT))
+        voice = acss.ACSS(voices.get(voiceType.get(DEFAULT)))
 
-        voice = acss.ACSS(rv)
         if key in [None, DEFAULT]:
             string = args.get('string', '')
-            if string.isupper():
+            if isinstance(string, str) and string.isupper():
                 voice.update(voices.get(voiceType.get(UPPERCASE)))
+        else:
+            override = voices.get(voicename)
+            if override and override.get('established', True):
+                voice.update(override)
 
         return [voice]
